#include "stdafx.H"
#include "AxControlDlg.H"

//------------------------------------------------------------------------------------------------
LRESULT CAXControlInsertDialog::OnInitDialog(UINT , WPARAM , LPARAM , BOOL& bHandled)
{
	HRESULT hr = m_spCatInfo.CoCreateInstance(CLSID_StdComponentCategoriesMgr);
	if( FAILED( hr ) ) 
	{
		EndDialog( IDCANCEL );
		return( TRUE );
	}

	PopulateList();
	bHandled = FALSE;
	return 0L;

}
//------------------------------------------------------------------------------------------------
LRESULT CAXControlInsertDialog::OnDestroy(UINT , WPARAM , LPARAM , BOOL& bHandled)
{
	ClearList();
	bHandled = FALSE;
	return 0L;
}
//------------------------------------------------------------------------------------------------

LRESULT CAXControlInsertDialog::OnListChange(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	HWND hWndList = GetDlgItem(IDC_CONTROLLIST);
	int nItem = ::SendMessage(hWndList, LB_GETCURSEL, 0, 0L);
	int nIndex = ::SendMessage(hWndList, LB_GETITEMDATA, nItem, 0L);
	GetClassServerPath(m_vecInfo[nIndex].m_clsid);
	return 0L;
}
//------------------------------------------------------------------------------------------------
LRESULT CAXControlInsertDialog::OnListItemDblClk(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	OnOK(wNotifyCode, wID, hWndCtl, bHandled);
	return 0L;
}
//------------------------------------------------------------------------------------------------

LRESULT CAXControlInsertDialog::OnOK(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	HWND hWndList = GetDlgItem(IDC_CONTROLLIST);
	int nItem = ::SendMessage(hWndList, LB_GETCURSEL, 0, 0L);
	int nIndex = ::SendMessage(hWndList, LB_GETITEMDATA, nItem, 0L);
	m_clsid = m_vecInfo[nIndex].m_clsid;

	EndDialog(wID);
	return 0;
}
//------------------------------------------------------------------------------------------------

LRESULT CAXControlInsertDialog::OnCancel(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled)
{
	EndDialog(wID);
	return 0;
}
//------------------------------------------------------------------------------------------------

HRESULT CAXControlInsertDialog::PopulateList()
{
	ClearList();
	CATID catid =CATID_Control;
	CComPtr<IEnumCLSID> spEnum;
	HRESULT hr = m_spCatInfo->EnumClassesOfCategories(1,&catid, 0, NULL, &spEnum );
	
	if( FAILED( hr ) )	return hr ;
	
	BOOL tDone			= FALSE;
	CLSID clsid			= CLSID_NULL;
	LPOLESTR	pszName = NULL;

	TCHAR		strName[1024];
	int nCount = 0;
	while( !tDone )
	{
		hr = spEnum->Next( 1, &clsid, NULL );
		if( hr == S_OK )
		{
			pszName = NULL;
			hr = ::OleRegGetUserType( clsid, USERCLASSTYPE_FULL, &pszName );
			if( SUCCEEDED( hr ) )
			{
				USES_CONVERSION;
				lstrcpy(strName,OLE2T(pszName));
				::CoTaskMemFree( pszName );
				pszName = NULL;
				
				HWND hWndList = GetDlgItem(IDC_CONTROLLIST);
				int iItem = ::SendMessage(hWndList, LB_ADDSTRING, 0, (LPARAM)strName);
				::SendMessage(hWndList, LB_SETITEMDATA, iItem, (LPARAM)nCount);
	
				InsertDlgInfo info;
				info.m_clsid = clsid;
				info.m_nIndex = nCount++;
				m_vecInfo.push_back(info);
			}
		}
		else
		{
			tDone = TRUE;
		}
	}

	return hr;
}
//------------------------------------------------------------------------------------------------

void CAXControlInsertDialog::ClearList()
{
	m_vecInfo.clear();
	
	//clear the listbox
	HWND hWndList = GetDlgItem(IDC_CONTROLLIST);
	::SendMessage(hWndList, LB_RESETCONTENT, 0, 0L);
}
//------------------------------------------------------------------------------------------------

HRESULT CAXControlInsertDialog::GetClassServerPath( REFCLSID clsid)
{
	HKEY hKey = NULL;
	HKEY hServerKey = NULL;
	OLECHAR szCLSID[64];
	LONG nResult = 0;
	ULONG nBytes = 0;
	DWORD dwType = 0;
	LPTSTR pszServerPath = NULL;

	StringFromGUID2( clsid, szCLSID, 64 );

	hKey = NULL;
	hServerKey = NULL;
	TCHAR psz[1024];
	lstrcpy(psz,_T("CLSID\\"));
	USES_CONVERSION;
	lstrcat(psz,OLE2T(szCLSID));
		
	nResult = RegOpenKeyEx( HKEY_CLASSES_ROOT, psz, 0, KEY_READ, &hKey );

	if( nResult != ERROR_SUCCESS )
	{
		return nResult;
	}

	nResult = RegOpenKeyEx( hKey, _T( "InprocServer32" ), 0, KEY_READ,&hServerKey );

	if( nResult != ERROR_SUCCESS )
	{
		nResult = RegOpenKeyEx( hKey, _T( "InprocHandler32" ), 0, KEY_READ,&hServerKey );

		if( nResult != ERROR_SUCCESS )
		{
			nResult = RegOpenKeyEx( hKey, _T( "LocalServer32" ), 0, KEY_READ,&hServerKey );
			if( nResult != ERROR_SUCCESS )
			{
			   RegCloseKey( hKey );
			   return nResult;
			}
		}
	}

	nBytes = 0;
	nResult = RegQueryValueEx( hServerKey, NULL, NULL, &dwType, NULL,&nBytes );
	if( (nResult != ERROR_SUCCESS) || (dwType != REG_SZ) )
	{
		RegCloseKey( hKey );
		return ( E_FAIL );
	}
	
	pszServerPath = LPTSTR( _alloca( nBytes ) );
	nResult = RegQueryValueEx( hServerKey, NULL, NULL, &dwType,LPBYTE( pszServerPath ), &nBytes );
	
	if( (nResult != ERROR_SUCCESS) || (dwType != REG_SZ) )
	{
		RegCloseKey( hKey );
		return( E_FAIL );
	}

	HWND hWndStatic = GetDlgItem(IDC_STATICPATH);
	::SetWindowText(hWndStatic,pszServerPath);

	RegCloseKey( hKey );
	hKey = NULL;
	RegCloseKey( hServerKey );
	hServerKey = NULL;

	return S_OK;

	}
//------------------------------------------------------------------------------------------------